/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2002 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
 
    You should have received a copy of the GNU General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to professionally license this library, the author
    can be reached at info@mohawksoft.com
*/ 

#define COUNT 5000000

#include "session.h"
#include "msessionapi.h"
#include <signal.h>

#define ONCE	0
#define VERBOSE	0

char *vars[]=
{
"test1", "fubar1",
"test2", "fubar2",
"test3", "fubar3",
"test4", "fubar4",
"test5", "fubar5",
"test6", "fubar6",
"test7", "fubar7",
"test1", "fubar7",
"test2", "fubar7",
"test3", "fubar7",
"test4", "fubar7",
"test5", "fubar7",
"test6", "fubar7",
"test7", "fubar7",
"test8", "fubar7",
"test9", "fubar7",
"testa", "fubar7",
"testb", "fubar7",
"testc", "fubar7",
"testd", "fubar7",
"teste", "fubar7",
"testf", "fubar7",
"testg", "fubar7",
"testh", "fubar7",
"testi", "fubar7",
NULL,NULL
};

void handleError(MSCONN conn,char *string)
{
	int err = msession_errno(conn);

	if(err != REQE_NOERROR)
	{
		fprintf(stderr, "%s [%d] %s\n", string, err, msession_errtext(conn));
		exit(0);
	}
}

FILE *createstatic(char *fname)
{
	FILE *file = fopen(fname, "wb");
	assert(file);

	for(int i=0; i < 512; i++)
	{
		fputc( (rand() % 26)+'A', file);
	}
	fputc( 0, file);

	fflush(file);
	return file;
}

char *getstatic()
{
	char fname[]="test.dat";
	FILE *file = fopen(fname,"rb");

	if(!file)
		file = createstatic(fname);

	assert(file);

	fseek(file, 0, SEEK_END);
	long flen = ftell(file);
	fseek(file, 0, SEEK_SET);

	char *data = (char*)malloc(flen)+1;

	assert(data);

	fread(data,1,flen, file);
	data[flen]=0;
	return data;
}

// Tests various functions in msession
void RegressionTest(MSCONN conn)
{
	Boolean fPass=FALSE;
	char *session;

	// get a random string.
	session = msession_randstr(conn, 64);

	if(!session)
		handleError(conn, "msession_randstr");

	// Print number of sessions.
	// printf("%d Sessions on line\n", msession_count(conn));
			
	// Create a session
	if(!msession_create(conn, session))
		handleError(conn, "msession_create");
	
	// Set some variables
	if(!msession_set(conn, session, "foo", "bar"))
		handleError(conn, "msesson_set");
	
	if(!msession_set(conn, session, "this", "that"))
		handleError(conn, "msesson_set");
	
	if(!msession_set(conn, session, "sid",session))
		handleError(conn, "msesson_set");
	
	// Test the counter
	msession_inc_int(conn, session, "counter");
	msession_inc_int(conn, session, "counter");
	msession_inc_int(conn, session, "counter");
	msession_inc_int(conn, session, "counter");

	// Read it back, make sure it works
	int n = msession_get_int(conn, session, "counter", 0);

	if(n != 4)
		printf("Counter failed should have been 4 when it was %d\n", n);

	// Get all the variables.
	char ** vars = msession_get_array(conn, session);

	if(!vars)
		handleError(conn, "msession_get_array");

	int num;
	for(num=0; vars[num]; num+=2)
		// printf("%s = %s\n", vars[num], vars[num+1])
		;
		

	// Now set them all back.
	if(!msession_set_array(conn, session, num/2, vars))
		handleError(conn, "msession_set_array");

	phmfree(vars);
		
	// Get on variable
	char *p = msession_get_text(conn, session, "foo", NULL);

	if(!p)
		handleError(conn, "msession_get");

	phmfree(p);

	// Find all sessions with counter
	vars = msession_find(conn, "counter", "4");

	if(!vars)
		handleError(conn, "msession_find");
	
	for(num=0, fPass=FALSE; vars[num]; num++)
	{
		// printf("%s\n", vars[num]);

		if(!strcmp(vars[num], session))
			fPass=TRUE;
	}

	if(!fPass)
	{
		printf("msession_find : Failed to find our session, system failed\n");
		exit(0);
	}
	phmfree(vars);
	// List the sessions.
	vars = msession_listvar(conn, "counter");

	if(!vars)
		handleError(conn, "msession_listvar");

	for(num=0, fPass=FALSE; vars[num]; num+=2)
	{
		// printf("%s = %s\n", vars[num+1], vars[num]);

		if(!strcmp(vars[num+1], session))
			fPass=TRUE;
	}
	if(!fPass)
	{
		printf("msession_listvar : Failed to find our session, system failed\n");
		exit(0);
	}

	phmfree(vars);
	msession_destroy(conn, session);

	phmfree(session);

}

extern unsigned int g_defReqFmt;

int main(int argc, char **argv)
{
	MSCONN conn;
	time_t start = time(0);
	time_t stop;
	char *session = NULL;
	int session_count=0;
	int opt=1;
	int optind=1;
	char *optarg;
	char *opthost="localhost";
	Boolean fRegress=FALSE;
	Boolean fPhpOnly=FALSE;
	Boolean fDelete = FALSE;
	int stopcount = COUNT;
	int sleeptime=0;
	int pageviews = 20;
	Boolean fLoopTest=TRUE;
	
	InitPhoenix(argc, argv);

	for (optind=1; 1; )
	{	opt = GetOption(argc,argv,"dh:rR:i:pP:s:?", &optarg, &optind);

		if(opt <= 0)
			break;
		switch (opt)
		{
			case 'd': // Destroy old sessions
				fDelete = TRUE;
				break;
			case 'h': // host name
				opthost = optarg;
				break;

			case 'r': // RegressionTest
				fRegress = TRUE;
				break;
			case 'R': // Regression ONLY!
				fLoopTest=FALSE;
				break;
				
			case 'p': // PHP only test
				fPhpOnly=TRUE;
				break;
			case 'i':
				stopcount = atoi(optarg);
				break;
			case 's': // Sleep time
				sleeptime = atoi(optarg)*1000;
				break;

			case 'P': // Page views
				pageviews = atoi(optarg);
				break;

			case '?': // Help
				printf("%s [options]\n", argv[0]);
				printf("-d, Destroy old sessions\n");
				printf("-h str, Host name\n");
				printf("-i, Interation count\n");
				printf("-p, PHP Performance test only\n");
				printf("-P num, Page views per session\n");
				printf("-r, Perform regression tests\n");
				printf("-s num, Sleep between iterations, num is in ms\n");
				exit(0);
		}
	}

	srand(time(0));

	g_defReqFmt = 0;

#ifdef POSIX
	signal(SIGPIPE, SIG_IGN);
#endif
	int count=0;

	int cbhost = strlen(opthost)+1;
	char *host = (char *)alloca(cbhost);
	int port = HostPort(host, opthost, 8086);

	conn = msession_connect(host, port);

	if(conn == NULL || !conn->StatOK())
	{
		fprintf(stderr, "Could not connect\n");
		exit(0);
	}

	// Perform a regression test
	if(fRegress)
		RegressionTest(conn);

	// If we are now load testing, skip regressions
	if(fPhpOnly)
		fRegress = FALSE;

	if(fLoopTest == FALSE)
		exit(0);

	// Get static data
	char *phpdata = getstatic();

	// Sync up clock
	while(start == time(0))
		;

	stop = start = time(0);


	do
	{
		msession_ping(conn);

		if(fRegress)
			RegressionTest(conn);
		
		if(!session)
		{
			// Number of times session is useed after creation
			// Think of this values as page views.
			session_count = 0;

			session = msession_uniq(conn,32);

			if(!session)
				handleError(conn,"msession_uniq");
		}
		session_count++;

		char * data = msession_getdata(conn, session);

		if(!data)
			data = phmstrdup(phpdata);
		else if(strcmp(data,phpdata))
		{
			int i;

			for(i=0; data[i]; i++)
			{
				if(data[i] != phpdata[i])
				{
					printf("Mismatch at %d %c != %c", i, data[i], phpdata[i]);
				}
			}
			printf("string mismatch\n%s\n%s\n", data, phpdata);
			assert(strcmp(data, phpdata)==0);
		}
				
		msession_setdata(conn, session, data);
		handleError(conn,"msession_setdata");
		if(!fPhpOnly)
		{
			msession_inc_int(conn, "globals", "count");
			handleError(conn,"msession_inc_int");
		}

		phmfree(data);

		count++;

		if(sleeptime)
			usleep(sleeptime);

		// Probability
		// Lets assume the average number of pages views
		// after creating a session is about 10 or 15. Some will 
		// go more and some will go fewer.
		if(session_count > (rand() % pageviews))
		{
			// User's never really destroy sessions!
			if(fDelete)
				msession_destroy(conn, session);
			phmfree(session);
			session = NULL;
		}

		if(count == stopcount)
			stop = time(0);
		else if(count > stopcount)
		{
			if(stop != time(0))
				break;	
		}

	}
	while(1);

	msession_disconnect(conn);
	handleError(conn,"msession_disconnect");

	stop = time(0);

	int cps = count/(stop-start);
	printf("connections per second %d\n", cps);
	return 0;
}
